-- UnsheathMove by Vohl --
-- Changelog:
-- V1.0 - First release
-- V1.1 - Added options screen and saved variables for default behaviour
-- V1.2 - Added new event for toggle autorun
-- V1.3 - Added New option and saved variable to enable whilst swimming
-- V1.4 - Added a Debug mode 
--      - Move forward and autorun triggers now only fire when NOT mounted 
--      - Toggle from keybind now only functions when NOT mounted  

-- initialise the defaults
local frame = CreateFrame("FRAME", "UnsheathMoveFrame");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");
local function eventHandler(self, event, ...)
	if UNSHEATHMOVE_DEFAULT == true then
		UnsheathMoveToggleOnOff = 'Enabled'
	else UnsheathMoveToggleOnOff = 'Disabled'
end
	if UNSHEATHMOVE_SWIMMING == true then
		UnsheathSwimming = 'Enabled'
	else UnsheathSwimming = 'Disabled'	
	end
end	
frame:SetScript("OnEvent", eventHandler);

UnsheathMoveDebug = 'Disbaled'

-- set up key bindings
BINDING_HEADER_UNSHEATHMOVE = "UnsheathMove"
BINDING_NAME_UNSHEATHMOVE = "Toggle"

-- Slash Commands
SLASH_UNSHEATHMOVE1 = '/um';
local function handler(param, editbox)
	if param == 'toggle' then
	ToggleSheath()
	UnsheathMoveToggleDefault()
  print('UnsheathMove is', UnsheathMoveToggleOnOff);
  elseif param == 'config' then
	InterfaceOptionsFrame_OpenToCategory("UnsheathMove")
	InterfaceOptionsFrame_OpenToCategory("UnsheathMove") -- sometimes the blizz options panel doesnt go to the right page.. this forces it to go to the right page
  elseif param == 'debug' then
		if UnsheathMoveDebug == 'Disbaled' then	UnsheathMoveDebug = 'Enabled'
		print('UnsheathMove Debug Mode Enabled')
		else UnsheathMoveDebug = 'Disbaled'
		print('UnsheathMove Debug Mode Disabled')
		end
  else
  print('-----------------------------------------');
  print('UnsheathMove is|cFFFFFF00', UnsheathMoveToggleOnOff, '|r');
  print('Possible options are:');
  print('/um toggle - to toggle on or off');
  print('/um config - to open the options screen');
  print('-----------------------------------------');
 end
end
SlashCmdList["UNSHEATHMOVE"] = handler;

--set up options screen
local UnsheathMoveDefaultText = "Enable the addon on startup"
local UnsheathMoveSwimmingText = "Enable when swimming"
local unsheathmove = CreateFrame("Frame", "unsheathmoveoptions", InterfaceOptionsFramePanelContainer)
unsheathmove:Hide()
unsheathmove.name = "UnsheathMove"
local title = unsheathmove:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", 16, -16)
title:SetText("UnsheathMove") 
InterfaceOptions_AddCategory(unsheathmove)

local btnUnsheathMoveDefault = CreateFrame("CheckButton", "UnsheathMoveDefaultButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveDefault:SetPoint("TOPLEFT", 16, -35)
btnUnsheathMoveDefault:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_DEFAULT = true
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_DEFAULT = nil
	end
end)
btnUnsheathMoveDefault:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_DEFAULT)
end)
local btnUnsheathMoveDefaultText = btnUnsheathMoveDefault:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveDefaultText:SetPoint("LEFT", btnUnsheathMoveDefault, "RIGHT", 0, 1)
btnUnsheathMoveDefaultText:SetText(UnsheathMoveDefaultText)

local btnUnsheathMoveSwimming = CreateFrame("CheckButton", "UnsheathMoveSwimmingButton", unsheathmove, "OptionsBaseCheckButtonTemplate")
btnUnsheathMoveSwimming:SetPoint("TOPLEFT", 16, -65)
btnUnsheathMoveSwimming:SetScript("OnClick", function(frame)
	if frame:GetChecked() then
		PlaySound("igMainMenuOptionCheckBoxOn")
		UNSHEATHMOVE_SWIMMING = true
		UnsheathSwimming = 'Enabled'
	else
		PlaySound("igMainMenuOptionCheckBoxOff")
		UNSHEATHMOVE_SWIMMING = nil
		UnsheathSwimming = 'Disbaled'
	end
end)
btnUnsheathMoveSwimming:SetScript("OnShow", function(frame)
	frame:SetChecked(UNSHEATHMOVE_SWIMMING)
end)
local btnUnsheathMoveSwimmingText = btnUnsheathMoveSwimming:CreateFontString(nil, "ARTWORK", "GameFontHighlight")
btnUnsheathMoveSwimmingText:SetPoint("LEFT", btnUnsheathMoveSwimming, "RIGHT", 0, 1)
btnUnsheathMoveSwimmingText:SetText(UnsheathMoveSwimmingText)

-- Debug mode - This prints the current state to the default chat frame everytime the addon triggers
function UnsheathMoveDebugMode()
	if GetSheathState() == 1 then UnsheathMoveDebugSheathState = 'Sheathed'
	end
	if GetSheathState() == 2 then UnsheathMoveDebugSheathState = 'Unsheathed'
	end
	print('UM:|cFFFFFF00', UnsheathMoveToggleOnOff, '|r:: ', 'SheathedState:|cFFFFFF00', UnsheathMoveDebugSheathState, '|r:: ', 'Mounted:|cFFFFFF00', IsMounted(), '|r::')
end


-- Toggle from keybind
function UnsheathMoveToggle()
	if not IsMounted() then 
		ToggleSheath()
		UnsheathMoveToggleDefault()
	end
	if UnsheathMoveDebug == 'Enabled' then UnsheathMoveDebugMode()
	end
end

-- Toggles default and checks against actual sheathstate to set variable
function UnsheathMoveToggleDefault()
		if UnsheathMoveToggleOnOff == 'Disabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Enabled'
		elseif UnsheathMoveToggleOnOff == 'Enabled' and GetSheathState() == 1
		then UnsheathMoveToggleOnOff = 'Disabled'
		else
			UnsheathMoveToggleOnOff = 'Disabled'
		end
end

--unsheath if enabled on moving forward
hooksecurefunc("MoveForwardStart", function()
	if GetSheathState() == 1 
	and (not IsSwimming() or (UnsheathSwimming == 'Enabled' and IsSwimming()))
	and not IsMounted()
	and UnsheathMoveToggleOnOff == 'Enabled'
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
	if UnsheathMoveDebug == 'Enabled' then UnsheathMoveDebugMode()
	end
end)

--unsheath if enabled and autorun is toggled
hooksecurefunc("ToggleAutoRun", function()
	if GetSheathState() == 1 
	and (not IsSwimming() or (UnsheathSwimming == 'Enabled' and IsSwimming()))
	and not IsMounted()
	and UnsheathMoveToggleOnOff == 'Enabled'
	then ToggleSheath()
	UnsheathMoveToggleOnOff = 'Enabled'
	end
	if UnsheathMoveDebug == 'Enabled' then UnsheathMoveDebugMode()
	end
end)